package cn.com.qmth.examcloud.support.cache;

import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Arrays;

import cn.com.qmth.examcloud.support.cache.bean.AppCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.web.cache.ObjectRedisCacheTrigger;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;

public class CacheHelper {

	private static RedisClient redisClient;

	private static ObjectRedisCacheTrigger objectRedisCacheTrigger;

	private static RedisClient getRedisClient() {
		if (null == redisClient) {
			redisClient = SpringContextHolder.getBean(RedisClient.class);
		}
		return redisClient;
	}

	private static ObjectRedisCacheTrigger getObjectRedisCacheTrigger() {
		if (null == objectRedisCacheTrigger) {
			objectRedisCacheTrigger = SpringContextHolder.getBean(ObjectRedisCacheTrigger.class);
		}
		return objectRedisCacheTrigger;
	}

	/**
	 * 加载并获取缓存对象
	 *
	 * @author WANGWEI
	 * @param keyPrefix
	 * @param propKeys
	 * @param c
	 * @param appName
	 * @param className
	 * @return
	 */
	private static <T> T get(String keyPrefix, Object[] propKeys, Class<T> c, String appName,
			String className) {

		String key = keyPrefix + StringUtils.join(Arrays.asList(propKeys), '_');

		T t = getRedisClient().get(key, c);
		if (null == t) {
			getObjectRedisCacheTrigger().fire(appName, className, propKeys);
			t = getRedisClient().get(key, c);
		}
		return t;
	}

	/**
	 * 获取APP
	 *
	 * @author WANGWEI
	 * @param appId
	 * @return
	 */
	public static AppCacheBean getApp(Long appId) {
		String key = "$_APP:" + appId;
		return getRedisClient().get(key, AppCacheBean.class);
	}

	/**
	 * 获取系统属性<br>
	 *
	 * @author WANGWEI
	 * @param propKey
	 * @return
	 */
	public static SysPropertyCacheBean getSysProperty(String propKey) {
		return get("$_SYS_PROP:", new String[]{propKey}, SysPropertyCacheBean.class,
				"EC-CORE-BASIC",
				"cn.com.qmth.examcloud.core.basic.service.cache.SystemPropertyCache");
	}

}
